use serde::{Deserialize, Serialize};
use yew::prelude::*;
use yew_router::prelude::*;

use components::*;
use pages::*;

mod components;
mod pages;
mod util;

#[derive(Clone, Routable, PartialEq)]
pub enum Route {
    #[at("/")]
    Home,
    #[at("/participants")]
    Participants,
    #[at("/participants/add")]
    AddParticipants,
    #[at("/times")]
    Times,
    #[at("/times/add")]
    AddTimes,
}

fn switch(route: Route) -> Html {
    match route {
        Route::Home => html! { <Home /> },
        Route::Participants => html! { <Participants /> },
        Route::AddParticipants => html! { <participants::AddParticipant /> },
        Route::Times => html! { <Times /> },
        Route::AddTimes => html! { <times::AddTime /> },
    }
}

#[derive(Serialize, Deserialize)]
#[serde(rename_all = "lowercase")]
pub enum MsgTypes {
    Users,
    Register,
    Message,
}

#[derive(Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
struct WebSocketMessage {
    message_type: MsgTypes,
    data_array: Option<Vec<String>>,
    data: Option<String>,
}

#[derive(PartialEq, Properties, Default)]
struct AppProps;

struct App;

impl Component for App {
    type Message = ();
    type Properties = AppProps;

    fn create(_ctx: &Context<Self>) -> Self {
        let wss = util::surrealdb::SurrealWebsocketService::new();

        let message = WebSocketMessage {
            message_type: MsgTypes::Register,
            data: Some("Hi!".to_string()),
            data_array: None,
        };

        if let Ok(_) = wss
            .tx
            .clone()
            .try_send(serde_json::to_string(&message).unwrap())
        {
            log::debug!("Message send successfully")
        }

        App
    }

    fn view(&self, _ctx: &Context<Self>) -> Html {
        html! {
            <BrowserRouter>
                <Navbar />
                <main>
                    <Switch<Route> render={switch} /> // <- must be child of <BrowserRouter>
                </main>
                <Footer />
            </BrowserRouter>
        }
    }
}

fn main() {
    wasm_logger::init(wasm_logger::Config::default());
    yew::Renderer::<App>::new().render();
}
